require 'spec_helper'

describe ForteRuby::API::Objects do
  subject { ForteRuby::API::Objects.new }
  let(:request_data) { double('request_data') }
  let(:single_class) { double('single_class') }
  let(:new_object) { double('new_object') }

  before do
    allow(subject).to receive(:single_class).and_return(single_class)
    allow(subject).to receive(:request_data).and_return(request_data)
  end

  describe '#build_from' do
    before { allow(single_class).to receive(:new).and_return(new_object) }

    it 'initializes object of single class' do
      expect(subject.build_from({})).to eq(new_object)
    end

    it 'sends attributes to new object' do
      expect(single_class).to receive(:new).with(name: 'rus', request_data: request_data, new: true)
      subject.build_from({name: 'rus'})
    end
  end

  describe '#all' do
    let(:response) { JSON.generate(results: [{ name: 'Bob' }]) }

    it 'sends request and parse response' do
      expect(subject)
        .to receive(:send_request)
        .with('http://asd.com/customers', :get)
        .and_yield(response)
        .and_return([new_object])
      expect(subject).to receive(:url).and_return('http://asd.com/customers')
      expect(subject).to receive(:parse_response).with(response)

      expect(subject.all).to eq([new_object])
    end
  end

  describe '#find_by_id' do
    let(:response) { JSON.generate(results: [{ name: 'Bob' }]) }

    it 'sends request and parse response' do
      expect(subject)
        .to receive(:send_request)
        .with('http://asd.com/customers/123', :get)
        .and_yield(response)
        .and_return([new_object])
      expect(subject).to receive(:url).and_return('http://asd.com/customers')
      expect(subject).to receive(:parse_response).with(response)

      expect(subject.find_by_id(123)).to eq([new_object])
    end
  end

  describe '#filter' do
    let(:response) { JSON.generate(results: [{ name: 'Bob' }]) }

    it 'sends request and parse response' do
      expect(subject)
        .to receive(:send_request)
        .with('http://asd.com/customers?filter=first_name+eq+Bob+and+last_name+eq+Ronald', :get)
        .and_yield(response)
        .and_return([new_object])
      expect(subject).to receive(:url).and_return('http://asd.com/customers')
      expect(subject).to receive(:parse_response).with(response)

      expect(subject.filter(first_name: 'Bob', last_name: 'Ronald')).to eq([new_object])
    end

    context 'if one filter passed' do
      it 'sends request with one filter' do
        expect(subject)
          .to receive(:send_request)
          .with('http://asd.com/customers?filter=first_name+eq+Bob', :get)
          .and_yield(response)
          .and_return([new_object])
        expect(subject).to receive(:url).and_return('http://asd.com/customers')
        expect(subject).to receive(:parse_response).with(response)

        expect(subject.filter(first_name: 'Bob')).to eq([new_object])
      end
    end
  end
end
